require_relative 'path_loader'

describe 'YubiRuby::HEX' do
  describe '#encode' do
    it 'should work as expected' do
      str = "ciao\n"
      hex = "6369616f0a"
      YubiRuby::HEX.encode(str).should == hex
    end
    
    it 'should be a reversible operation' do
      str = "ciao\n"
      hex = YubiRuby::HEX.encode(str)
      YubiRuby::HEX.decode(hex).should == str
    end
  end
  
  describe '#decode' do
    it 'should work as expected' do
      str = "ciao\n"
      hex = "6369616f0a"
      YubiRuby::HEX.decode(hex).should == str
    end
    
    it 'should be a reversible operation' do
      hex = "6369616f0a"
      str = YubiRuby::HEX.decode(hex)
      YubiRuby::HEX.encode(str).should == hex
    end
    
    it 'should raise NoHexEncodedError if decoding non Hex string' do
      lambda { YubiRuby::HEX.decode('a')  }.should raise_error(YubiRuby::NoHexEncodedError)
      lambda {YubiRuby::HEX.decode('zebr') }.should raise_error(YubiRuby::NoHexEncodedError)
      lambda {YubiRuby::HEX.decode('!af5') }.should raise_error(YubiRuby::NoHexEncodedError)
      lambda {YubiRuby::HEX.decode('0x12ff') }.should raise_error(YubiRuby::NoHexEncodedError)
    end
  end
  
  describe '#hex?' do
    it 'should recognize valid hex' do
      YubiRuby::HEX.hex?('7af5').should == true
      YubiRuby::HEX.hex?('12fF').should == true
    end
    
    it 'should recognize invalid hex' do
      YubiRuby::HEX.hex?('a').should == false
      YubiRuby::HEX.hex?('zebr').should == false
      YubiRuby::HEX.hex?('!af5').should == false
      YubiRuby::HEX.hex?('0x12ff').should == false
    end
  end
end
