require_relative 'path_loader'

describe 'YubiRuby::ModHex' do
  
  it 'should be a reversible operation' do
    str = "test"
    hex = "ifhgieif"
    YubiRuby::ModHex.decode(YubiRuby::ModHex.encode(str)).should == str
    YubiRuby::ModHex.encode(YubiRuby::ModHex.decode(hex)).should == hex
  end
  
  describe '#encode' do
    it 'should work as expected' do
      str = "test"
      hex = "ifhgieif"
      YubiRuby::ModHex.encode(str).should == hex
    end
    
    it "should double the string lenght" do
      str = '1234zz567890'
      str.size.should == 12
      str.modhex_encode.size.should == 24
    end

  end
  
  describe '#decode' do
    it 'should work as expected' do
      str = "test"
      hex = "ifhgieif"
      YubiRuby::ModHex.decode(hex).should == str
    end
    
    it 'should raise NoModHexEncodedError if decoding non ModHex string' do
      lambda { YubiRuby::ModHex.decode('a')  }.should raise_error(YubiRuby::NoModHexEncodedError)
      lambda {YubiRuby::ModHex.decode('zebr') }.should raise_error(YubiRuby::NoModHexEncodedError)
      lambda {YubiRuby::ModHex.decode('!af5') }.should raise_error(YubiRuby::NoModHexEncodedError)
      lambda {YubiRuby::ModHex.decode('0x12ff') }.should raise_error(YubiRuby::NoModHexEncodedError)
    end
    
    it "should halves the string lenght" do
      otp = 'dbkutdgrnlvrhdiregebvkkrgtuefjru'
      otp.size.should == 32
      otp.modhex_decode.size.should == 16
    end
  end
  
  describe '#hex?' do
    it 'should recognize valid hex' do
      YubiRuby::ModHex.modhex?('ifhgieif').should == true
      YubiRuby::ModHex.modhex?('ichkicichv').should == true
    end
    
    it 'should recognize invalid hex' do
      YubiRuby::ModHex.modhex?('a').should == false
      YubiRuby::ModHex.modhex?('zebr').should == false
      YubiRuby::ModHex.modhex?('!af5').should == false
      YubiRuby::ModHex.modhex?('0x12ff').should == false
    end
  end
end

describe 'String' do
  describe '#modhex_decode' do
    it "should call YubiRuby::ModHex#decode once" do
      otp = 'dbkutdgrnlvrhdiregebvkkrgtuefjru'
      YubiRuby::ModHex.should_receive(:decode).once.with(otp)
      YubiRuby::ModHex.should_not_receive(:encode)
      otp.modhex_decode
    end
  end
  
  describe '#modhex_encode' do
    it "should call YubiRuby::ModHex#encode once" do
      str = 'ciaociao ciao'
      YubiRuby::ModHex.should_receive(:encode).once.with(str)
      YubiRuby::ModHex.should_not_receive(:decode)
      str.modhex_encode
    end
  end
end
