class TestFakeYubikey < Test::Unit::TestCase
  def test_init
    key = YubiRuby::FakeYubikey.new
    assert_equal("000000000000", key.uid )
    assert_equal(0, key.session_use)
    assert_equal(1, key.session_counter)
  end
  
  def test_otp
    key = YubiRuby::FakeYubikey.new
    aes = key.set_random_key
    uid = key.set_random_uid
    assert(aes.hex?)
    assert(uid.hex?)
    validator = YubiRuby::Yubikey.new aes
    assert(validator.parse(key.otp))
    assert_equal(validator.uid, uid)
    assert_equal(validator.session_use, key.session_use)
    assert_equal(validator.session_counter, key.session_counter)
  end
  
  def test_generate_random_uid
    key = YubiRuby::FakeYubikey.new
    assert_equal(key.set_random_uid, key.uid)
  end
  
  def test_generate_random_key
    key = YubiRuby::FakeYubikey.new
    assert_equal(key.set_random_key, key.key)
  end
    
  def test_generate_specific_otp_and_fake_yubikey
    aes = 'df5b9d7591b0b3c2819f7a0dd7d1547d'
    session = 53
    tsl = 31716
    tsh = 106
    counter = 0
    random = 30172
    crc = 58822
    uid = "6d8dec078841"
    check_otp = 'rfechkcdkjrflgrjlirdgivnedbbtcik'
    result = YubiRuby::FakeYubikey.generate_specific_otp_and_fake_yubikey(aes, uid, session, 
      tsl, tsh, counter, random)
    mycrc = YubiRuby::CRC16.calculate(result[1].to_s[0...-2]).ones_complement 16
    
    
    YubiRuby::Yubikey.new(aes).parse(result[0]) 
    assert_equal(aes,result[1].key)
    assert_equal(session,result[1].session_counter)
    assert_equal(tsl,result[1].timestamp_low)
    assert_equal(tsh,result[1].timestamp_high)
    assert_equal(counter,result[1].session_use)
    assert_equal(random,result[1].random)
    assert_equal(crc,result[1].crc)
    assert_equal(uid,result[1].uid)
    assert_equal(check_otp,result[0])
  end
  
  def test_session_use_overflow
    key = YubiRuby::FakeYubikey.new
    aes = key.set_random_key
    uid = key.set_random_uid
    session = key.session_counter
    1.upto(2**8-1) do |i|
      key.otp 
      assert_equal(session, key.session_counter) 
      assert_equal(i, key.session_use)
    end
    key.otp
    assert_equal(session+1, key.session_counter)
    assert_equal(0, key.session_use)
  end
 
#   ruby(71000) malloc: *** error for object 0x101aa6e70: 
#     incorrect checksum for freed object - object was probably modified after being freed.  
#  def test_session_counter_overflow
#    key = YubiRuby::FakeYubikey.new
#    aes = key.set_random_key
#    uid = key.set_random_uid
#    session = key.session_counter
#    assert_equal(1, session)
#    2.upto(YubiRuby::FakeYubikey::Data::SESSION_COUNTER_OVERFLOW) do |i|
#      key.init 
#    end
#    assert_raise YubiRuby::YubikeySessionCounterOverflow do
#      key.init
#    end
#  end
end